@@ -12,7 +12,6 @@ use pyo3::{
 use rand::Rng;
 use regex::Regex;
 use rr_util::{
-    compact_data::TinyBoundedBitvec,
     py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
     pycall, python_error_exception, simple_from,
     tensor_util::{
@@ -118,7 +117,7 @@ pub trait SpecTrait: fmt::Debug + ToPyObject {
     fn serialize(&self) -> Result<Option<String>>;
     fn get_device_dtype_override(
         &self,
-        device_dtypes: &[TorchDeviceDtypeOp],
+        _device_dtypes: &[TorchDeviceDtypeOp],
     ) -> Option<Result<TorchDeviceDtypeOp>> {
         None
     } // returning None means use normal device_dtype inheritance. if Some, then error if dtypes are incorrect and dtype if correct
@@ -132,7 +131,7 @@ pub struct GeneralFunctionShapeInfo {
     #[pyo3(get, set)]
     pub num_non_batchable_output_dims: u8,
     #[pyo3(get, set)]
-    pub input_batchability: TinyBoundedBitvec,
+    pub input_batchability: Vec<bool>,
 }
 
 #[pymethods]
@@ -147,7 +146,7 @@ impl GeneralFunctionShapeInfo {
     pub fn new(
         shape: Shape,
         num_non_batchable_output_dims: u8,
-        input_batchability: TinyBoundedBitvec,
+        input_batchability: Vec<bool>,
     ) -> Self {
         Self {
             shape,
@@ -457,7 +456,7 @@ impl SpecTrait for GeneralFunctionSetDDSpec {
         Ok(GeneralFunctionShapeInfo {
             shape: shapes[0].clone(),
             num_non_batchable_output_dims: 0,
-            input_batchability: vec![false].try_into().unwrap(),
+            input_batchability: vec![true].try_into().unwrap(),
         })
     }
     fn get_device_dtype_override(
@@ -617,7 +616,6 @@ impl SpecTrait for Box<PyWrap> {
                 .call_method1(
                     py,
                     "get_device_dtype_override",
-                    // PyShape here is important!
                     PyTuple::new(
                         py,
                         device_dtypes
@@ -718,7 +716,7 @@ pub struct GeneralFunction {
     name: Option<String>,
     #[pyo3(get)]
     pub num_non_batchable_output_dims: u8,
-    pub input_batchability: TinyBoundedBitvec,
+    pub input_batchability: Vec<bool>,
 }
 
 impl GeneralFunction {
@@ -751,18 +749,17 @@ impl GeneralFunction {
         let mut out = Self {
             nodes,
             spec,
-            name: Default::default(),
+            name,
             info: Default::default(),
             num_non_batchable_output_dims,
             input_batchability,
         };
-        out.name = out.auto_name(name);
         out.info.shape = shape; // set shape so we only compute once
-        out.init_info()
+        out.initial_init_info()
     }
 
     pub fn is_batchable(&self) -> bool {
-        self.input_batchability.into_iter().any(|x| x)
+        self.input_batchability.iter().any(|x| *x)
     }
 }
 
@@ -814,7 +811,7 @@ impl CircuitNode for GeneralFunction {
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         let num_batchable_axes = self.info().rank() as u8 - self.num_non_batchable_output_dims;
-        zip(&self.nodes, self.input_batchability)
+        zip(&self.nodes, &self.input_batchability)
             .map(|(child, batchable)| {
                 if !batchable {
                     vec![None; child.info().rank()]
@@ -838,25 +835,23 @@ impl CircuitNode for GeneralFunction {
 impl CircuitNodeAutoName for GeneralFunction {
     const PRIORITY: OperatorPriority = OperatorPriority::Function {};
 
-    fn auto_name(&self, name: Option<String>) -> Option<String> {
+    fn auto_name(&self) -> Option<String> {
         // Never any parenthesis, so we don't even care checking if we need to add some
         // but check out children_names_with_maybe_paren if you change the syntax of the general function
-        name.or_else(|| {
-            if self.children().any(|x| x.name().is_none()) {
-                None
-            } else {
-                Some(
-                    self.spec.name().clone()
-                        + "("
-                        + &self
-                            .children()
-                            .map(|x| Self::shorten_child_name(x.name().unwrap()))
-                            .collect::<Vec<String>>()
-                            .join(", ")
-                        + ")",
-                )
-            }
-        })
+        if self.children().any(|x| x.name().is_none()) {
+            None
+        } else {
+            Some(
+                self.spec.name().clone()
+                    + "("
+                    + &self
+                        .children()
+                        .map(|x| Self::shorten_child_name(x.name().unwrap()))
+                        .collect::<Vec<String>>()
+                        .join(", ")
+                    + ")",
+            )
+        }
     }
 }
 
@@ -881,10 +876,6 @@ impl GeneralFunction {
         parse_string: String,
         name: Option<String>,
     ) -> Result<Self> {
-        Python::with_gil(|_py| {
-            println!("{parse_string}");
-        });
-
         if parse_string.contains(" at ") {
             let mut split = parse_string.splitn(2, " at ");
             let name = convert_op_str_to_format_err(split.next(), &parse_string)?;
@@ -1144,14 +1135,14 @@ impl GeneralFunctionSpecTester {
             );
         }
 
-        if input_batchability.into_iter().all(|x| !x) {
+        if input_batchability.iter().all(|x| !*x) {
             // if none batchable, we don't have any tests to run
             return Ok((true, false));
         }
         let current_num_batch_dims = shape.len() - num_non_batchable_output_dims as usize;
 
-        for (shape, is_batch) in
-            std::iter::once((&shape, true)).chain(shapes.iter().zip(input_batchability))
+        for (shape, &is_batch) in
+            std::iter::once((&shape, &true)).chain(shapes.iter().zip(&input_batchability))
         {
             if is_batch && shape.len() < current_num_batch_dims {
                 bail!(
@@ -1166,7 +1157,7 @@ impl GeneralFunctionSpecTester {
             .chain(
                 shapes
                     .iter()
-                    .zip(input_batchability)
+                    .zip(&input_batchability)
                     .filter_map(|(s, is_batch)| is_batch.then(|| &s[..current_num_batch_dims])),
             )
             .collect();
@@ -1186,8 +1177,8 @@ impl GeneralFunctionSpecTester {
 
             let new_shapes: Vec<Shape> = shapes
                 .iter()
-                .zip(input_batchability)
-                .map(|(s, is_batch)| {
+                .zip(&input_batchability)
+                .map(|(s, &is_batch)| {
                     if is_batch {
                         batch_shape
                             .iter()
@@ -1228,7 +1219,7 @@ impl GeneralFunctionSpecTester {
             );
             }
 
-            if new_info.input_batchability != input_batchability {
+            if &new_info.input_batchability != &input_batchability {
                 bail!(
                     "{}changed input_batchability when only batching was changed\n{}\n{}",
                     prefix,
@@ -1312,8 +1303,8 @@ impl GeneralFunctionSpecTester {
                     .function(
                         &tensors
                             .iter()
-                            .zip(input_batchability)
-                            .map(|(x, b)| if b { run_idx(x.clone()) } else { x.clone() })
+                            .zip(&input_batchability)
+                            .map(|(x, &b)| if b { run_idx(x.clone()) } else { x.clone() })
                             .collect::<Vec<_>>(),
                         &Default::default(),
                     )
