@@ -13,12 +13,11 @@ use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
     python_error_exception,
-    rearrange_spec::{OpSize, RearrangeSpec},
+    rearrange_spec::{OpSize, RInts, RearrangeSpec},
     sv,
     tensor_util::{Slice, TensorAxisIndex, TensorIndex},
-    util::EinsumAxes,
+    tu8v,
 };
-use smallvec::SmallVec;
 use thiserror::Error;
 
 use crate::nest::einsum_flatten;
@@ -48,13 +47,12 @@ pub fn rewrite_cum_to_circuit_of_cum(
             let nb_axis_before = start_axis as u8;
             let nb_axis_after = (cumulant.rank() - start_axis - node.rank()) as u8;
             let new_spec_axis = spec.next_axis();
-            let before_axis: SmallVec<[SmallVec<[u8; 8]>; 4]> = (new_spec_axis
-                ..(new_spec_axis + nb_axis_before))
-                .map(|i| sv![i as u8])
+            let before_axis: RInts = (new_spec_axis..(new_spec_axis + nb_axis_before))
+                .map(|i| tu8v![i as u8])
                 .collect();
-            let after_axis: SmallVec<[SmallVec<[u8; 8]>; 4]> = ((new_spec_axis + nb_axis_before)
+            let after_axis: RInts = ((new_spec_axis + nb_axis_before)
                 ..(new_spec_axis + nb_axis_before + nb_axis_after))
-                .map(|i| sv![i as u8])
+                .map(|i| tu8v![i as u8])
                 .collect();
             let new_input_ints = [
                 before_axis.clone(),
@@ -238,15 +236,7 @@ fn cum_of_einsum_to_einsum_of_cum_raw(
                 let (prod, _new_cums) = kappa_term(fused_per, &on_sub_cumulant)?;
 
                 let operated_on_prod = Einsum::try_new(
-                    vec![(
-                        prod.rc(),
-                        einsum
-                            .input_axes()
-                            .cloned()
-                            .collect::<Vec<EinsumAxes>>()
-                            .concat()
-                            .into(),
-                    )],
+                    vec![(prod.rc(), einsum.input_axes().flatten().cloned().collect())],
                     out_nums.clone().into(),
                     Some("perm".to_owned()), /* name=Cumulant.get_name(*new_cums, getter=namer) + " perm",  # naming to ensure no duplicates */
                 )?; // TODO: better name
